// Data set URI
const csvUrl =
  'https://raw.githubusercontent.com/Rastanne/Activity7_Datasets/main/Weatherdata_Seattle.csv';

// Define the dimensions of the chart
const margin = {
    top: 30,
    right: 320,
    bottom: 120,
    left: 50,
  },
  width = 960 - margin.left - margin.right,
  height = 500 - margin.top - margin.bottom;

// Parse the date / time
const parseTime = d3.timeParse('%Y');

// Set the ranges
const x = d3.scaleTime().range([0, width]);
const y = d3.scaleLinear().range([height, 0]);

// Define the svg canvas
const svg = d3
  .select('svg')
  .attr('width', width + margin.left + margin.right)
  .attr('height', height + margin.top + margin.bottom)
  .append('g')
  .attr(
    'transform',
    'translate(' + margin.left + ',' + margin.top + ')',
  );

// Set up the color scale
const color = d3.scaleOrdinal(d3.schemeCategory10);

// Get the data
d3.csv(csvUrl).then(function (data) {
  // Format the data
  data.forEach(function (d) {
    d.date = parseTime(d.Date);
    d.Max_TemperatureC = +d.Max_TemperatureC;
    d.Mean_TemperatureC = +d.Mean_TemperatureC;
    d.Min_TemperatureC = +d.Min_TemperatureC;
  });

  // Stack the data
  const stack = d3
    .stack()
    .keys([
      'Max_TemperatureC',
      'Mean_TemperatureC',
      'Min_TemperatureC',
    ])
    .order(d3.stackOrderNone)
    .offset(d3.stackOffsetNone);

  const layers = stack(data);

  // Scale the range of the data
  x.domain(
    d3.extent(data, function (d) {
      return d.date;
    }),
  );
  y.domain([
    0,
    d3.max(layers, (layer) => d3.max(layer, (d) => d[1])),
  ]);
  // Add the area layers
  svg
    .selectAll('.layer')
    .data(layers)
    .enter()
    .append('path')
    .attr('class', 'layer')
    .attr(
      'd',
      d3
        .area()
        .x(function (d, i) {
          return x(d.data.date);
        })
        .y0(function (d) {
          return y(d[0]);
        })
        .y1(function (d) {
          return y(d[1]);
        }),
    )
    .style('fill', function (d, i) {
      return color(i); // Use a color scale of your choice here
    });

  // Add the X Axis
  svg
    .append('g')
    .attr('transform', 'translate(0,' + height + ')')
    .call(d3.axisBottom(x));

  // Add the Y Axis
  svg.append('g').call(d3.axisLeft(y));

  // X-axis label
  svg
    .append('text')
    .attr(
      'transform',
      'translate(' +
        width / 2 +
        ' ,' +
        (height + margin.top + 20) +
        ')',
    )
    .style('text-anchor', 'middle')
    .text('Year');

  // Y-axis label
  svg
    .append('text')
    .attr('transform', 'rotate(-90)')
    .attr('y', 0 - margin.left)
    .attr('x', 0 - height / 2)
    .attr('dy', '1em')
    .style('text-anchor', 'middle')
    .text('Temparature');

  // Add the legend
  const legend = svg
    .selectAll('.legend')
    .data(layers)
    .enter()
    .append('g')
    .attr('class', 'legend')
    .attr('transform', function (d, i) {
      return (
        'translate(' +
        (width + margin.right / 2) +
        ',' +
        i * 20 +
        ')'
      );
    });

  legend
    .append('rect')
    .attr('width', 18)
    .attr('height', 18)
    .style('fill', function (d, i) {
      return color(i);
    });

  legend
    .append('text')
    .attr('x', 24)
    .attr('y', 9)
    .attr('dy', '.35em')
    .style('text-anchor', 'start')
    .text(function (d) {
      return d.key;
    });

  // Create tooltip
  const tooltip = d3
    .select('body')
    .append('div')
    .attr('class', 'tooltip')
    .style('position', 'absolute')
    .style('opacity', 0)
    .style('background', 'white')
    .style('border', 'solid 1px black')
    .style('padding', '10px')
    .style('pointer-events', 'none'); // Make sure the tooltip doesn't interfere with mouse events

  svg
    .selectAll('.layer')
    .attr('pointer-events', 'all')
    .on('mouseover', function (d) {
      tooltip.style('display', 'block');
    })
    .on('mousemove', function (event, d) {
      // Use d3.pointer() to get the current pointer position
      const [pointerX] = d3.pointer(event, this);
      // Use the x-scale to find the date corresponding to the pointer position
      const xDate = x.invert(pointerX);
      // Attempt to find the data for that year
      const dataYear = data.find(
        (h) => h.date.getFullYear() === xDate.getFullYear(),
      );

      // Display the tooltip
      tooltip
        .html(
          `<strong>Date:</strong> ${xDate.getFullYear()}<br>
           <strong>Max_TemperatureC:</strong> ${dataYear.Max_TemperatureC}<br>
           <strong>Mean_TemperatureC:</strong> ${dataYear.Mean_TemperatureC}<br>
           <strong>Min_TemperatureC:</strong> ${dataYear.Min_TemperatureC}`,
        )
        .style('left', `${event.pageX + 10}px`)
        .style('top', `${event.pageY + 10}px`)
        .style('opacity', 1);
    })
    .on('mouseout', function () {
      tooltip.style('opacity', 0); // Hide tooltip on mouse out
    });
});
